import uuid
from datetime import datetime
from enum import Enum
from typing import Any, Dict, List, Optional, Union

from bson.objectid import ObjectId
from pydantic import UUID4, BaseModel, Field

from msaDocModels import sdu, wdc
from msaDocModels.sdu import (
    SDUAttachment,
    SDUContent,
    SDUData,
    SDUEmail,
    SDULanguage,
    SDUText,
)


class TenantIdInput(BaseModel):
    """
    Input model with tenant_id

    Attributes:
        tenant_id: tenant's uuid.
    """

    tenant_id: UUID4


class TextInput(BaseModel):
    """
    Input model with input_text

    Attributes:
        input_text: input text.
    """

    input_text: Union[str, List, Dict]


class DocumentInput(TextInput):
    """
    Input document model

    Attributes:
        document_id: optional uuid for document.
    """

    document_id: Optional[UUID4]


class SentencesInput(BaseModel):
    """
    Input model with sentences

    Attributes:
        document_id: optional uuid for document.
        sentences: list of sentences
    """

    document_id: Optional[UUID4]
    sentences: List[str]


class DocumentIds(BaseModel):
    """
    Ids of documents from mail model

    Attributes:
        document_ids: list of uuids.
    """

    document_ids: List[str]


class DocumentLangInput(DocumentInput):
    """
    Input document model made over SDULanguage. Default language english

    Attributes:
        language: object SDULanguage.
    """

    language: SDULanguage = SDULanguage(code="en", lang="english")


class SPKLanguageInput(DocumentInput):
    """
    Input model to detect language.

    Attributes:
        hint_languages: language hint for analyzer. 'ITALIAN' or 'it' boosts Italian;see LANGUAGES for known languages.
        hint_encoding: encoding hint for analyzer. 'SJS' boosts Japanese; see ENCODINGS for all known encodings.
        sentence_detection: turn on/off language detection by sentence.
        get_vectors: turn on/off return of sentence vectors.
        is_plain_text: if turned off, and HTML is provided as input, detection will skip HTML tags,
            expand HTML entities and detect HTML <lang ...> tags.
        is_short_text: turn on to get the best effort results (instead of unknown) for short text.
    """

    hint_languages: str = ""
    hint_encoding: str = ""
    sentence_detection: bool = True
    get_vectors: bool = True
    is_plain_text: bool = True
    is_short_text: bool = False


class SPKLanguageDTO(sdu.SDULanguageDetails):
    """DTO, representing the result of service language."""


class TextWithParagraphsGet(BaseModel):
    """
    Schema representing the result of paragraph segmentation.

    Attributes:
        paragraphs: list of SDUParagraph.
    """

    paragraphs: List[sdu.SDUParagraph]


class TextWithSentencesGet(BaseModel):
    """
    Schema representing the result of sentences segmentation.

    Attributes:
        sentences: list of SDUSentence.
    """

    sentences: List[sdu.SDUSentence]


class TextWithPagesGet(BaseModel):
    """
    Schema representing the result of pages segmentation.

    Attributes:
        pages: list of SDUPage.
    """

    pages: List[sdu.SDUPage]


class SPKSegmentationInput(BaseModel):
    """
    Input model to detect Segmentation

    Attributes:
        document_id: optional uuid for document.
        input_text: input_text.
        language: SDULanguage object for this text.

    """

    document_id: Optional[UUID4]
    input_text: Union[str, List[str], Dict[int, str]]
    language: SDULanguage = SDULanguage(code="en", lang="english")


class SPKSegmentationDTO(BaseModel):
    """
    DTO, representing the result of service segmentation. Only one attribute will be non-empty.

    Attributes:
        pages: list of SDUPage.
        paragraphs: list of SDUParagraph.
        sentences: list of SDUSentence.
    """

    pages: List[sdu.SDUPage] = []
    paragraphs: List[sdu.SDUParagraph] = []
    sentences: List[sdu.SDUSentence] = []


class SPKTextCleanInput(DocumentInput):
    """Data input model for Text Clean."""


class SPKTextCleanDTO(BaseModel):
    """
    DTO, representing the result of service text clean.

    Attributes:
        text: cleaned text.
    """

    text: str


class SPKDataCleanAIInput(BaseModel):
    """
    Data input model for Text AI Clean.

    Attributes:

        data: List of nested dictionaries
        keys: The keys  which need to clean
        language: default is german
    """

    data: List[Dict[str, Dict[str, Any]]]
    keys: List[str] = []
    language: SDULanguage = SDULanguage(code="de", lang="german")


class SPKDataCleanAIDTO(BaseModel):
    """
    DTO, representing the result of service ai text clean.

    Attributes:

        data: LList of nested dictionaries
    """

    data: List[Dict[str, Dict[str, Any]]]


class SPKSentimentInput(DocumentInput):
    """Data input model for Sentiment."""


class SPKSentimentDTO(BaseModel):
    """
    DTO, representing the result of service Sentiment.

    Attributes:
        neg:Negativity score of the text.
        neu: Neutrality score of the text.
        pos: Positivity score of the text.
        compound: Compound score of the text.
        error: None if there is no errors, otherwise contains description of the error.
    """

    neg: Optional[float]
    neu: Optional[float]
    pos: Optional[float]
    compound: Optional[float]
    error: Optional[str]


class SPKPhraseMiningInput(DocumentLangInput):
    """Data input model for Phrase mining."""


class SPKPhraseMiningDTO(BaseModel):
    """
    DTO, representing the result of Phrase mining.

    Attributes:
        phrases: Nested list of most common phrases in the provided sentence(s)
    """

    phrases: List[Union[List, List[Union[str, int]]]]


class SPKWeightedKeywordsDTO(BaseModel):
    """
    DTO, representing the result of service Keywords.

    Attributes:
        keywords:  List of keywords and/or keyphrases.
    """

    keywords: List[Union[List, List[Union[str, int]]]]


class SPKExtractKeywordsInput(BaseModel):
    """
    Data input model for ExtractKeywords.

    Attributes:
        data: extended input text by InputKeyKeys, have the len as input.
        algorithms: which algorithms use for extract. Can be list of ["yake", "bert", "bert_vectorized", "tf_idf]
        keys: which keys need to extract
        language: default is german
    """

    data: List[Dict[str, Dict[str, Any]]]
    algorithms: List[str] = ["yake", "bert"]
    keys: List[str] = []
    language: SDULanguage = SDULanguage(code="de", lang="german")


class SPKExtractKeywordsDTO(BaseModel):
    """
    DTO, representing the result of service Keywords.

    Attributes:
        data: extended input text by InputKeyKeys, have the len as input.
    """

    data: List[Dict[str, Dict[str, Any]]]


class SPKSummaryInput(DocumentLangInput):
    """
    Data input model for Summary.

    Attributes:
        sum_ratio: Coefficient.
        sentences_count: Amount of sentences.
        lsa: Algorithm
        corpus_size: Coefficient
        community_size: Coefficient
        cluster_threshold: Coefficient
    """

    sum_ratio: float = 0.2
    sentences_count: int = 15
    lsa: bool = False
    corpus_size: int = 5000
    community_size: int = 5
    cluster_threshold: float = 0.65


class SPKStatisticsInput(DocumentLangInput):
    """Data input model for Statistics."""


class SPKStatisticsDTO(sdu.SDUStatistic):
    """DTO, representing the result of service Statistics."""


class SPKSummaryDTO(wdc.WDCItem):
    """DTO, representing the result of service Summary."""


class SPKNotaryInput(DocumentInput):
    """
    Data input model for Notary.

    Attributes:
        city: default city to check.
    """

    city: str = "Bremen"


class SPKNotary(BaseModel):
    """Detected Notary Pydantic Model."""

    sid: Optional[str]
    last_name: Optional[str]
    first_name: Optional[str]
    zip_code: Optional[str]
    city: Optional[str]
    office_city: Optional[str]
    official_location: Optional[str]
    address: Optional[str]
    additional_address: Optional[str]
    title: Optional[str]
    phone: Optional[str]
    complete_name_with_official_location: Optional[str]
    local_city: str = "Bremen"
    is_local_city: bool


class SPKNotaryWinnerDTO(SPKNotary):
    """DTO, representing the result of service Notary."""


class SPKCountry(BaseModel):
    """Detected Country Pydantic Model."""

    name: str
    official: str
    currencies: Dict[str, Dict[str, str]]
    capital: List[str]
    region: str
    subregion: str
    languages: Dict[str, str]
    latlng: List[int]
    flag: str
    calling_codes: List[str]


class SPKCompany(BaseModel):
    """Detected Company Pydantic Model."""

    rank: int
    company: str
    employees: str
    change_in_rank: str
    industry: str
    description: str
    revenue: str
    revenue_change: str
    profits: str
    profit_change: str
    assets: str
    market_value: str


class SPKCity(BaseModel):
    """Detected City Pydantic Model."""

    name: str
    country: str
    latlng: List[float]


class SPKTaxonomyCitiesDTO(BaseModel):
    """
    DTO, representing the result of service Taxonomy Cities.

    Attributes:

        cities: List of SPKCities.
        cities_winner: winner object SPKCity.
    """

    cities: List[SPKCity]
    cities_winner: Optional[SPKCity]


class SPKTaxonomyCountriesDTO(BaseModel):
    """
    DTO, representing the result of service Taxonomy Countries.

    Attributes:

        countries: List of SPKCountries.
        countries_winner: winner object SPKCountry.
    """
    countries: List[SPKCountry]
    countries_winner: Optional[SPKCountry]


class SPKTaxonomyCompaniesDTO(BaseModel):
    """
    DTO, representing the result of service Taxonomy Companies.

    Attributes:

        companies: List of SPKCompanies.
        companies_winner: winner object SPKCompany.
    """

    companies: List[SPKCompany]
    companies_winner: Optional[SPKCompany]


class SPKTaxonomyDTO(
    SPKTaxonomyCountriesDTO, SPKTaxonomyCompaniesDTO, SPKTaxonomyCitiesDTO
):
    """DTO, representing the result of service Taxonomy."""


class SPKTaxonomyInput(DocumentInput):
    """Data input model for Taxonomy."""


class AutoMLStatus(BaseModel):
    """
    Pydantic model to receive/send service status for pub/sub.

    Attributes:

        info: Service status.
        id: UUID model identifier.
        path: The path where model is located
        model_data: train columns, features, etc
    """

    info: str
    id: Optional[uuid.UUID]
    path: Optional[str]
    model_data: Optional[Dict]


class SPKProfileInput(BaseModel):
    """
    Pydantic model to generate a profile report based on data

    Attributes:
        title: Title of HTML representation.
        data: List of data.
        missing_diagrams: Settings related with the missing data section and the visualizations it can include.
        vars: Vars to provide another settings.
        correlations: Settings regarding correlation metrics and thresholds.
        sort: Default sorting.
        progress_bar: If True will display a progress bar.
        minimal: Minimal mode is a default configuration with minimal computation.
        explorative: Explorative mode.
        sensitive: Sensitive mode.
        dark_mode: Select a dar theme.
        orange_mode: Select a orange theme.

    """

    title: str
    html: Dict = {}
    missing_diagrams: Dict = {}
    correlations: Dict = {}
    vars: Dict = {}
    data: List[Dict[str, Any]]
    sort: str = "ascending"
    progress_bar: bool = False
    minimal: bool = False
    explorative: bool = False
    sensitive: bool = False
    dark_mode: bool = False
    orange_mode: bool = False


class SPKProfileDTO(BaseModel):
    """
    Pydantic model of Profile HTML representation
    """


class SPKBuildModelInput(BaseModel):
    """
    Model that contains input data for building a machine learning model.

    Attributes:

        model_name: The name of the model.
        data: The input data to be used for model training.
        target_columns: Column names representing the target variable(s) to be predicted.
        train_columns: Column names to be used for model training.
        text_features: Column names representing text features to be used for text processing.
        ignore_features: Column names representing features to be ignored during model training.
        categorical_features: Column names representing categorical features.
        date_features: Column names representing date features.
        numeric_features: Column names representing numeric features.
        ordinal_features: Dictionary of column names representing ordinal features and their categories.
        multiplier: Multiplier used for increasing the size of the training data using synthetic samples.
        session_id: Seed value used for reproducibility.
        remove_outliers: Flag indicating whether to remove outliers from the data.
        budget_time_minutes: Maximum time in minutes allowed for model training.
        included_engines: List of machine learning models to be used for model training.
        use_gpu: Flag indicating whether to use GPU for model training.
        fold: Number of folds for cross-validation.
        tuning_iterations: Number of iterations for hyperparameter tuning.
        create_metadata: Flag indicating whether to create model metadata
        webhook_url: url to custom HTTP back requests.
    """
    model_name: str = "kim_pipeline"
    data: List[Dict[str, Any]]
    target_columns: List[str] = ["IMPULSART", "IMPULSKATEGORIE"]
    train_columns: List[str] = []
    text_features: List[str] = ["SACHVERHALT", "SACHVERHALT_KEYWORDS"]
    ignore_features: List[str] = []
    categorical_features: List[str] = []
    date_features: List[str] = []
    numeric_features: List[str] = []
    ordinal_features: dict[str, list] = {}
    multiplier: int = 5
    session_id: int = 123
    remove_outliers: bool = False
    budget_time_minutes: float = 3.0
    included_engines: List[str] = ["svm", "nb", "ridge", "rf", "dt"]
    use_gpu = False
    fold: int = 7
    tuning_iterations: int = 7
    create_metadata = False
    webhook_url: Optional[str]


class SPKInferenceInput(BaseModel):
    """
     Pydantic model for get inference data.

    Attributes:

        path: The path where model is located.
        data: Profile html representation.
    """

    path: str
    data: List[Dict[str, Any]]


class SPKInferenceDTO(BaseModel):
    """
    Pydantic model, provided merged inference data.

    Attributes:

        inference: Raw data with inference data.
    """

    inference: List[Dict[str, Any]]


class ProcessStatus(BaseModel):
    """
    Workflow status

        Attributes:

        number: number of status
        timestamp: time when number was changes
    """

    number: int = 0
    timestamp: str = str(datetime.utcnow())


class SPKDBBaseDocumentInput(BaseModel):
    """
    Document fields for input.

    Attributes:

        uid: document uid
        name: document name.
        mimetype: mimetype.
        path: path to file.
        layout_file_path: path to layout file.
        debug_file_path: path to debug file.
        readorder_file_path: path to rearorder file.
        folder: folder name.
        group_uuid: group identifier.
        size_bytes: size in bytes.
        is_file: file or not.
        wfl_status: wfl status.
        import_status: import status.
        user: user name.
        date: date.
        runtime_s: runtime in sec.
        tags: list of tags.
        language: language.
        needs_update: need update or not.
        data: data.
        project_code: project code.
        npages: count of pages.
        content: content.
        metadata: metadata.
        description: discription.
        status: document status
        text: text.
        file: file.
        sdu: Dict of sdu objects.
    """

    uid: str
    name: str
    mimetype: str = "text/plain"
    path: str = ""
    layout_file_path: str = ""
    debug_file_path: str = ""
    readorder_file_path: str = ""
    folder: str = ""
    group_uuid: str = ""
    size_bytes: int = 0
    is_file: bool = False
    wfl_status: List = []
    import_status: str = "new"
    user: str = ""
    date: str = ""
    runtime_s: float = 0.0
    tags: Optional[Dict] = {}
    language: Optional[SDULanguage] = None
    needs_update: bool = False
    data: Optional[SDUData] = None
    project_code: str = ""
    npages: int = 0
    content: Optional[SDUContent] = None
    metadata: Dict = {}
    description: str = ""
    text: str = ""
    file: Dict = {}
    sdu: Dict = {}
    status: ProcessStatus = ProcessStatus()
    status_history: List[ProcessStatus] = [ProcessStatus()]


class PyObjectId(ObjectId):
    """
    Converts ObjectId to string.
    """

    @classmethod
    def __get_validators__(cls):
        """
        Generator to return validate method.
        """
        yield cls.validate

    @classmethod
    def validate(cls, v):
        """
        Validates Object ID.

        Parameters:

             v: value to validate.

        Returns:

            Object ID with specified value.

        Raises:

            ValueError if Object ID does not pass validation.
        """
        if not ObjectId.is_valid(v):
            raise ValueError("Invalid objectid")
        return ObjectId(v)

    @classmethod
    def __modify_schema__(cls, field_schema):
        field_schema.update(type="string")


class MongoId(BaseModel):
    """
    MongoDB _id field.

    Attributes:

        id: The id of element in mongo.
    """

    id: PyObjectId = Field(default_factory=PyObjectId, alias="_id")

    class Config:
        allow_population_by_field_name = True
        json_encoders = {ObjectId: str}


class SPKDBBaseDocumentDTO(SPKDBBaseDocumentInput, MongoId):
    """
    Document fields for output.
    """


class BaseInfo(BaseModel):
    """
    Base info for AI stuff.

    Attributes:

        version: version identifier.
        description: description.
        datetime: datetime.
        inherited: inherited or not.
        active: active or not.
        name: object name.
    """

    version: str
    description: str
    datetime: datetime
    inherited: bool
    active: bool
    name: str


class SPKUpdateAI(BaseModel):
    """
    Update ai fields.

    Attributes:

        version: version identifier.
        description: description.
        datetime: datetime.
        inherited: inherited or not.
        active: active or not.
        name: object name.
    """

    version: Optional[str]
    description: Optional[str]
    datetime: Optional[datetime]
    inherited: Optional[bool]
    active: Optional[bool]
    name: Optional[str]


class SPKLearnsetDataInput(BaseInfo):
    """
    AI learnset input.

    Attributes:

        learnsets: list of learnset objects.
    """

    learnsets: List[Dict]


class SPKTestsetDataInput(BaseInfo):
    """
    AI testset input.

    Attributes:

        testsets: list of testsets objects.
    """

    testsets: List[Dict]


class SPKTaxonomyDataInput(BaseInfo):
    """
    AI taxonomy input.

    Attributes:

        taxonomies: list of taxonomies objects.
    """

    taxonomies: List[Dict]


class SPKModelDataInput(BaseInfo):
    """
    AI model input.

    Attributes:

        model: model object.
    """

    model: Dict


class SPKTestsetDataDTO(SPKTestsetDataInput, MongoId):
    """
    AI testset output.
    """


class SPKLearnsetDataDTO(SPKLearnsetDataInput, MongoId):
    """
    AI learnset output.
    """


class SPKModelDataDTO(SPKModelDataInput, MongoId):
    """
    AI model output.
    """


class SPKTaxonomyDataDTO(SPKTaxonomyDataInput, MongoId):
    """
    AI taxonomy output.
    """


class SPKConversionInput(BaseModel):
    """
    Model that contains inference data along with filenames to use for XLSX conversion.

    Attributes:

        filenames: list of filenames that files should be saved as
        inference: inference data, first key means sheet name for XLSX file
    """

    filenames: List[str]
    inference: List[Dict[str, Dict[str, Any]]]


class SPKHTMLConverterResponse(BaseModel):
    """
    Response from converter

    Attributes:

        metadata: metadata from file
        txt_content: SDUText object
    """

    metadata: Dict
    txt_content: SDUText


class SPKEmailConverterResponse(BaseModel):
    """
    Matching pydantic models with fields in the db.

    Attributes:
        content_attachments: list of SDUAttachments.
        txt_content: SDUText.
        msg: SDUEmail.
        content_unzipped_files: object of
    """

    content_attachments: List[SDUAttachment]
    txt_content: SDUText
    msg: SDUEmail
    content_unzipped_files: Optional[List[SPKHTMLConverterResponse]]


class FieldName(str, Enum):
    """
    Matching pydantic models with fields in the db.

    Attributes:
        TestsetDataInput: name of testset input model.
        LearnsetDataInput: name of learnset input model.
        ModelDataInput: name of model input model.
        TaxonomyDataInput: name of taxonomy input model.
    """

    TestsetDataInput = "testset"
    LearnsetDataInput = "learnset"
    ModelDataInput = "model"
    TaxonomyDataInput = "taxonomy"



# -*- coding: utf-8 -*-
from typing import Any, Optional

from lxml.html.clean import Cleaner


async def sanitize(dirty_html: Any) -> Optional[str]:
    """Clean/Sanitize HTML using lxml.html.clean.Cleaner

    Cleans the following:

    * Removes any ``<meta>`` tags
    * Removes any embedded objects (flash, iframes)
    * Removes any ``<link>`` tags
    * Removes any style tags.
    * Removes any processing instructions.
    * Removes any style attributes.  Defaults to the value of the ``style`` option.
    * Removes any ``<script>`` tags.
    * Removes any Javascript, like an ``onclick`` attribute. Also removes stylesheets as they could contain Javascript.
    * Removes any comments.
    * Removes any frame-related tags
    * Removes any form tags
    * Removes Tags that aren't *wrong*, but are annoying.  ``<blink>`` and ``<marquee>``
    * Remove any tags that aren't standard parts of HTML.
    * Remove any attributes which are not frozenset(['src', 'color', 'href', 'title', 'class', 'name', 'id']),
    * Remove Tags ('span', 'font', 'div'), their content will get pulled up into the parent tag.

    Args:
        dirty_html: Any, usually a html str

    Returns:
         clean_html: Optional[str] cleaned html
    """
    cleaner = Cleaner(
        page_structure=True,
        meta=True,
        embedded=True,
        links=True,
        style=True,
        processing_instructions=True,
        inline_style=True,
        scripts=True,
        javascript=True,
        comments=True,
        frames=True,
        forms=True,
        annoying_tags=True,
        remove_unknown_tags=True,
        safe_attrs_only=True,
        safe_attrs=frozenset(["src", "color", "href", "title", "class", "name", "id"]),
        remove_tags=("span", "font", "div"),
    )

    return cleaner.clean_html(dirty_html)



import dataclasses
import datetime
from dataclasses import dataclass
from typing import Generic, Type, TypeVar

from genson import SchemaBuilder
from pydantic.types import UUID

_T = TypeVar("_T")

try:
    import orjson as json
except:
    try:
        import ujson as json

    except:
        import json

        json.__version__ = ""


import uuid
from typing import List, Optional, Union

from fastapi.encoders import jsonable_encoder
from pydantic import BaseModel


class titem(BaseModel):
    item_me: str = "Stefan"


class test(BaseModel):
    info: str = "some information"
    test_list: List[str] = ["a", "list"]
    test_item: titem = titem()
    test_item_list: List[titem] = [titem()]
    test_optional_item: Optional[titem] = None
    test_date: datetime.datetime = datetime.datetime.utcnow()


class Metadata(BaseModel):
    msg_type: Optional[str] = None
    msg_id: UUID = uuid.uuid4()
    sender_id: UUID = uuid.uuid4()
    created_at: datetime.datetime = datetime.datetime.utcnow()
    json_module: str = json.__name__
    json_version: str = json.__version__
    metaclass: str = ""


class MSAAPIMessage:
    def __init__(self, metaClass: _T = Metadata, **kwargs):
        self.__dict__ = kwargs
        self.metadata: metaClass = metaClass()
        self.metadata.metaclass = metaClass.__name__

    def toDict(self) -> dict:
        return jsonable_encoder(self.__dict__)

    def fromDict(self, msg: dict):
        self.__dict__ = msg

    def toMsg(self) -> str:
        classes = {}
        for k, v in self.__dict__.items():
            classes[k] = v.__class__.__name__
            # print("TOMSG:",k,v, type(v), v.__class__.__name__)
        self.__dict__["classes"] = classes
        return (
            json.dumps(self, default=lambda x: jsonable_encoder(x.__dict__))
            .decode("utf8")
            .replace("'", '"')
        )

    def fromMsg(self, message: str):
        self.__dict__ = json.loads(message)

    def fromMsgToClasses(self, message: str):
        self.fromMsg(message=message)
        if "metaclass" in self.metadata.keys():
            mc = globals()[self.metadata["metaclass"]]
            self.metadata = mc(**self.__dict__.pop("metadata"))
        if "classes" in self.__dict__.keys():
            for k, v in self.__dict__["classes"].items():
                try:
                    mv = globals()[v]
                    self.__dict__[k] = mv(**self.__dict__[k])
                except:
                    if not isinstance(self.__dict__[k], dict):
                        mv = v
                        try:
                            self.__dict__[k] = type(mv)(self.__dict__[k])
                        except:
                            pass

    def schema_json(self) -> str:
        builder = SchemaBuilder()
        builder.add_object(self.toDict())
        return builder.to_json()


if __name__ == "__main__":
    m = MSAAPIMessage(
        info="some information",
        mylist=["a", "list"],
        test=test(),
    )

    print("m todict:", m.toDict())
    print("m tomsg:", m.toMsg())
    print("m str:", m.__str__())
    print("m api:", jsonable_encoder(m))
    print("m rep:", m.__repr__())
    print("m cls:", m.__class__)
    print("schema:", m.schema_json())

    print("old instance test", m.test)
    print("old instance test", type(m.test))

    mya = MSAAPIMessage()
    print("New instance after blank initialize", mya.toMsg())
    mya.fromMsg(m.toMsg())
    print("New instance after fromMsg", mya.toMsg())
    print("New instance meta", mya.metadata)
    print("New instance type of mya.test", type(mya.test))

    mya.fromMsgToClasses(m.toMsg())
    print("New instance after fromMsgToClasses", mya.toMsg())
    print("New instance meta", mya.metadata)
    print("New instance type of mya.test", type(mya.test))



# -*- coding: utf-8 -*-

"""Module for the Semantic Document Understanding - Content

"""
import html
import os
from typing import Dict, List, Optional, Tuple

from pydantic import BaseModel

from msaDocModels.utils.htmlutils import sanitize


def getCRLF() -> str:
    """get's the OS Environment Variable for ``CR_LF``.
    Default: ``\\n``
    """
    ret: str = os.getenv("CR_LF", "\n")
    return ret


def getSentenceSeperator() -> str:
    """get's the OS Environment Variable for ``SENTENCE_SEPARATOR``.
    Default: `` `` (Space/Blank)
    """
    ret: str = os.getenv("SENTENCE_SEPARATOR", " ")
    return ret


def getCRParagraph() -> str:
    # CR_PARAGRAPH
    """get's the OS Environment Variable for ``CR_PARAGRAPH``.
    Default: ``\\n\\n``
    """
    ret: str = os.getenv("CR_PARAGRAPH", "\n\n")
    return ret


class SDUPageImage(BaseModel):
    """Page Image Pydantic Model.

    Storing the information about the Image representation of a Page.
    """

    id: int = -1  # ID = Page Index.
    filepath_name: str = ""  # Filepath to the image on filesystem storage.
    height: float = 0.0  # Image Height.
    width: float = 0.0  # Image Width.
    dpi: float = 0.0  # Picture DPI Resolution.
    format: str = ""  # Image Format (png, jpg etc.).
    mode: str = ""  # Image Mode.
    layout: List = []  # Image Layout Information."""

    class Config:
        orm_mode = False


class SDUEmail(BaseModel):
    """Parsed EMail Pydantic Model."""

    msg_id: str = ""
    msg_from: str = ""
    msg_to: str = ""
    msg_cc: str = ""
    msg_bcc: str = ""
    msg_subject: str = ""
    msg_sent_date: str = ""
    msg_body: str = ""
    seg_body: str = ""  # Segmented Body (Signature, etc.)
    seg_sign: str = ""
    msg_sender_ip: str = ""
    msg_to_domains: str = ""
    msg_received: List = []
    msg_reply_to: str = ""
    msg_timezone: str = ""
    msg_headers: Dict = {}

    class Config:
        orm_mode = False


class SDUDetailLanguage(BaseModel):
    """Detailed Language Pydantic Model."""

    multiple: bool = False
    reliable: bool = False
    bytes: int = -1
    details: Optional[Tuple] = tuple()
    vectors: Optional[Tuple] = tuple()

    class Config:
        orm_mode = False


class SDULanguage(BaseModel):
    """Detected Language Pydantic Model."""

    code: str = "unknown"  # Short de, en etc.
    lang: str = "unknown"  # Language name like german.
    reliable: bool = False  # is the detected result reliable.
    proportion: int = -1  # Proportion of the text in this language.
    bytes: int = -1  # Bytes of the text in this language.
    confidence: float = -1  # Confidence from 0.01 to 1.0.
    winner: Optional[str] = None  # Selected overall Winner

    class Config:
        orm_mode = False


class SDULanguageDetails(SDULanguage):
    details: Optional[List] = list()  # Details of the top 3 detected languages.


class SDUStatistic(BaseModel):
    """Text Statistics Pydantic Model."""

    avg_character_per_word: float = 0
    avg_letter_per_word: float = 0
    avg_sentence_length: float = 0
    avg_syllables_per_word: float = 0
    avg_sentence_per_word: float = 0
    difficult_words: int = 0
    lexicon_count: int = 0
    long_word_count: int = 0
    reading_time_s: float = 0
    sentence_count: int = 0
    paragraph_count: int = 0
    reading_ease_score: float = 0
    reading_ease: str = ""
    grade: float = 0
    smog: float = 0
    coleman: float = 0
    reading_index: float = 0
    reading_score: float = 0
    write_formula: float = 0
    fog: float = 0
    standard: str = ""
    crawford: float = 0
    gulpease_index: float = 0
    osman: float = 0

    class Config:
        orm_mode = False


class SDUSentence(BaseModel):
    id: int = -1
    text: str = ""
    xpos: List[str] = []
    upos: List[str] = []
    tokens: List[str] = []

    class Config:
        orm_mode = False


class SDUPDFElement(BaseModel):
    line_id: int = -1
    span_id: int = -1
    flags: int = 0
    bold: bool = False
    italic: bool = False
    font: str = ""
    fontsize: float = 0.0
    color: int = 0


class SDUParagraph(BaseModel):
    id: int = -1
    sort: int = -1
    nsen: int = 0
    semantic_type: str = "text"
    section: str = "body"
    size_type: str = "body"
    sentences: List[SDUSentence] = []
    sentences_en: List[SDUSentence] = []
    clean: str = ""
    lang: SDULanguage = SDULanguage()
    elements: List[SDUPDFElement] = []

    class Config:
        orm_mode = False

    def hasText(self) -> bool:
        return len(self.sentences) > 0

    def getText(self) -> str:
        return self.getTextNoLF()

    def getTextNoLF(self) -> str:
        ret = ""
        for sen in self.sentences:
            ret += sen.text + " "
        return ret

    def getTextLF(self) -> str:
        ret = ""
        for sen in self.sentences:
            ret += sen.text + getCRLF()
        return ret


class SDUText(BaseModel):
    raw: str = ""
    clean: str = ""
    html_content: str = ""
    structured_content: Dict = {}
    lang: SDULanguage = SDULanguage()
    paragraphs: List[SDUParagraph] = []

    class Config:
        orm_mode = False


class SDUPage(BaseModel):
    id: int = -1
    npar: int = 0
    input: str = ""
    has_en: bool = False
    text: SDUText = SDUText()

    class Config:
        orm_mode = False

    def hasText(self):
        return len(self.text.paragraphs) > 0

    def getTextDefault(self):
        return self.getTextNoLF()

    def getTextNoLF(self):
        ret = ""
        for par in self.text.paragraphs:
            for sen in par.sentences:
                ret += sen.text + " "
        return ret

    def getTextNoLF_EN(self):
        ret = ""
        for par in self.text.paragraphs:
            for sen in par.sentences_en:
                ret += sen.text + " "
        return ret

    def getTextNoLF_Paragraph(self):
        ret = ""
        for par in self.text.paragraphs:
            for sen in par.sentences:
                ret += sen.text + getSentenceSeperator()
            ret += getCRParagraph()
        return ret

    def getTextLF(self, space_before_lf: bool = False):
        ret = ""
        for par in self.text.paragraphs:
            for sen in par.sentences:
                ret += sen.text
                if space_before_lf:
                    ret += getSentenceSeperator()
                ret += getCRLF()
        return ret

    def getTextLF_Paragraph(self, space_before_lf: bool = False):
        ret = ""
        for par in self.text.paragraphs:
            for sen in par.sentences:
                ret += sen.text
                if space_before_lf:
                    ret += getSentenceSeperator()
                ret += getSentenceSeperator()
            ret += getCRParagraph()
        return ret

    def getAllSentencesTextListLF(self):
        ret = []
        for par in self.text.paragraphs:
            for i, sen in enumerate(par.sentences):
                txt = sen.text
                if i == len(par.sentences) - 1:
                    txt = txt + getCRParagraph()
                else:
                    txt = txt + getCRLF()
                ret.append(txt)
        return ret

    def getTextForNLP(self):
        ret = []

        for par in self.text.paragraphs:
            txt = ""
            for sen in par.sentences:
                txt += sen.text.replace("\n", "") + "\n\n"
            if len(txt) > 1:
                ret.append(txt)
        return ret

    def getTextForDisplay(self):
        ret = ""
        for par in self.text.paragraphs:
            txt: str
            if par.semantic_type.__contains__("list"):
                txt = par.getText()
                ret += txt + getCRLF()
            else:
                txt = par.getText()
                if par.semantic_type.__contains__(
                    "head"
                ) or par.semantic_type.__contains__("title"):
                    ret += getCRLF() + txt + getCRLF()
                elif len(par.sentences) > 1:
                    ret += txt + getCRParagraph()
                else:
                    ret += txt + getCRLF()
        ret = ret.replace(getCRParagraph() + getCRLF(), getCRParagraph())
        return ret

    def getAllSentencesTextList(self):
        ret = []

        for par in self.text.paragraphs:
            txt = ""
            if par.semantic_type.__contains__("list"):
                txt = par.getTextLF()
            else:
                txt = par.getText()
            ret.append(txt)
        return ret

    def getAllSentencesTextListNoTableAndLists(self):
        ret = []

        for par in self.text.paragraphs:
            txt = ""
            if par.semantic_type.__contains__("table"):
                pass
            elif par.semantic_type.__contains__("list"):
                pass
            elif par.semantic_type.__contains__("image_text"):
                pass
            elif par.semantic_type.__contains__("imagetext"):
                pass
            elif par.semantic_type.__contains__("image"):
                pass
            elif par.semantic_type.__contains__("figure"):
                pass
            elif par.section.__contains__("footer"):
                pass
            elif par.semantic_type.__contains__("title"):
                pass
            elif par.semantic_type.__contains__("headline"):
                pass
            else:
                for i, sen in enumerate(par.sentences):
                    if len(txt) > 0:
                        txt += " "
                    txt = txt + sen.text  # getCRParagraph()
                ret.append(txt)
        return ret

    def getAllSentencesTextList_en(self):
        ret = []
        for par in self.text.paragraphs:
            for i, sen in enumerate(par.sentences_en):
                txt = sen.text
                if i == len(par.sentences_en) - 1:
                    txt = txt + getCRParagraph()
                else:
                    txt = txt + getSentenceSeperator()
                ret.append(txt)
        return ret

    def setInput(self, inputText: str):
        self.input = inputText


class SDUVersion(BaseModel):
    version: str = ""
    creation_date: str = ""

    class Config:
        orm_mode = False


class SDULearnset(BaseModel):
    # dict entry is the class, list are the train entrys for this class
    version: str = ""
    text: Dict = {}
    nlu: Dict = {}
    nlp: Dict = {}
    emb: Dict = {}
    vec_words: Dict = {}
    vec_sent: Dict = {}

    def set_version(self, version: str):
        self.version = version

    def reset(self):
        self.text.clear()
        self.nlu.clear()
        self.nlp.clear()
        self.emb.clear()
        self.vec_words.clear()
        self.vec_sent.clear()

    class Config:
        orm_mode = False


class SDUData(BaseModel):
    npages: int = 0  #
    stats: SDUStatistic = SDUStatistic()  #
    pages: List[SDUPage] = []
    converter: List[str] = []
    email: SDUEmail = SDUEmail()  # email header
    text: SDUText = SDUText()  # parsed/body only
    images: List[SDUPageImage] = []

    class Config:
        orm_mode = False

    def addPagePreProcessing(self, pagepre: SDUPage):
        if pagepre:
            self.pages.append(pagepre)
            self.npages = len(self.pages)
            pagepre.page = self.npages

    def escaped(self):
        return html.escape(self.text.html_content)

    async def sanitized(self):
        return await sanitize(self.text.html_content)


class SDUBBox(BaseModel):
    x0: float = -1
    y0: float = -1
    x1: float = -1
    y1: float = -1

    class Config:
        orm_mode = False


class SDUElement(BaseModel):
    id: int
    start: int = -1
    end: int = -1

    class Config:
        orm_mode = False


class SDUAttachment(BaseModel):
    id: str = ""
    name: str = ""
    path: str = ""
    meta: Dict = {}
    text: SDUText = SDUText()
    charset: str = ""
    encoding: str = ""
    disposition: str = ""
    content_type: str = ""
    binary: bool = False
    payload: str = ""
    status: str = ""

    class Config:
        orm_mode = False


class SDUDimensions(BaseModel):
    id: int = -1
    height: float = 0.0
    width: float = 0.0
    factor_x: float = 0.0
    factor_y: float = 0.0
    rotation: int = 0

    class Config:
        orm_mode = False


class SDUFonts(BaseModel):
    id: int = -1
    fontsizes: Dict = {}
    fonts: List = []
    avg_fontsize: int = 14
    small_fontsize: int = 10000

    class Config:
        orm_mode = False


class SDULayout(BaseModel):
    id: int = -1
    dimensions: SDUDimensions = SDUDimensions()
    fonts: SDUFonts = SDUFonts()
    texttrace: List = []
    images: List = []
    drawings: List = []
    blocks: List[tuple] = []
    bjson: Dict = {}
    columns: List[SDUBBox] = []
    rows: List[SDUBBox] = []
    layouts: List = []
    header: SDUBBox = SDUBBox()
    body: SDUBBox = SDUBBox()
    footer: SDUBBox = SDUBBox()
    margin_left: SDUBBox = SDUBBox()
    margin_right: SDUBBox = SDUBBox()

    class Config:
        orm_mode = False


class SDUContent(BaseModel):
    attachments: List[SDUAttachment] = []
    layouts: List[SDULayout] = []

    class Config:
        orm_mode = False



# -*- coding: utf-8 -*-
from typing import Optional

from pydantic import BaseModel


class MSAHealthDefinition(BaseModel):
    """Health Service Definition"""

    path: str = "/healthcheck"
    """ Path in URL for the API"""
    interval: Optional[int] = 60
    """ Interval the Healtchcheck Thread sleeps and checks"""
    enabled: bool = True
    """ Is the healtcheck enabled True/False"""


class MSAHealthMessage(BaseModel):
    """Health Pydantic Response Service Message"""

    healthy: bool = False
    message: Optional[str]
    error: Optional[str]



# -*- coding: utf-8 -*-

import html
from typing import Dict, List, Optional, Tuple

from pydantic import BaseModel


class WDCPosition(BaseModel):  # Used for entity
    id: int = 0
    pageid: int = -1
    paraid: int = -1  #
    senid: int = -1
    s: int = -1
    e: int = -1


class WDCWord(BaseModel):
    id: int = -1
    text: str = ""
    lemma: str = ""
    misc: Optional[str] = None
    pos: str = ""
    type: Optional[str] = ""
    morph: str = ""
    head: int = -1
    label: str = ""
    deps: str = ""
    role: str = ""


class WDCToken(BaseModel):
    id: int = -1
    position: WDCPosition = WDCPosition()
    nwords: int = 0
    text: str = ""
    misc: Dict = {}
    ner: str = ""
    density: float = 0.0
    words: List[WDCWord] = []

    def addWord(self, wrd: WDCWord):
        if wrd:
            wrd.id = len(self.words)
            self.words.append(wrd)
            self.nwords = len(self.words)


class WDCTriple(BaseModel):
    id: int = -1
    position: WDCPosition = WDCPosition()
    score: float = 0.0
    subject: Dict = {}
    predicate: Dict = {}
    object: Dict = {}
    utterance: str = ""
    perspective: Dict = {}


class WDCItem(BaseModel):
    id: int = -1
    score: float = 0.0
    text: str = ""
    type: str = ""
    pos: List[Tuple] = []
    pos_text: List[str] = []


class WDCMeaning(BaseModel):  # Used for entity
    id: int = -1
    positions: List[WDCPosition] = []
    description: str = ""
    type: str = ""
    synset: str = ""
    offset: int = 0
    ssid: str = ""
    lexname: str = ""
    tree: List[str] = []
    tree_names: List[str] = []
    root_tree: List[str] = []
    root_tree_names: List[str] = []
    domains: List[str] = []
    frames: List[str] = []
    frames_id: List[int] = []
    frames_set: List[str] = []
    frames_set_id: List[int] = []
    root: str = ""
    lemmas: List[str] = []
    sees: List[str] = []
    hypernyms: List[str] = []
    hyponyms: List[str] = []
    holonyms: List[str] = []
    meronyms: List[str] = []
    similar: List[str] = []
    entailments: List[str] = []
    name: str = ""
    misc: Dict = {}
    hits: int = 1
    token: str = ""
    pos: str = ""
    context: List[str] = []

    def addPosition(self, pos: WDCPosition):
        if pos:
            self.positions.append(pos)

    def processSynset(self, syn, type: str):
        if not syn:
            return
        self.type = type
        # print("processSynset", self.token, syn)

        self.pos = syn.pos()
        self.synset = syn.name()
        self.offset = syn.offset()
        self.ssid = str(self.offset).zfill(8) + "-" + syn.pos()
        # lemmas
        self.lemmas = syn.lemma_names()
        # lexname
        ln = str(syn.lexname()).split(".")
        if len(ln) > 0:
            self.lexname = ln[1]
        else:
            self.lexname = str(syn.lexname())

        # similar
        lst = syn.similar_tos()
        self.similar = [entry.name() for entry in lst]
        # hypernyms
        lst = syn.hypernyms()
        self.hypernyms = [entry.name() for entry in lst]
        # hyponyms
        lst = syn.hyponyms()
        self.hyponyms = [entry.name() for entry in lst]
        # holonyms
        lst = syn.part_holonyms()
        self.holonyms = [entry.name() for entry in lst]
        # meronyms
        lst = syn.part_meronyms()
        self.meronyms = [entry.name() for entry in lst]
        # entailments
        lst = syn.entailments()
        self.entailments = [entry.name() for entry in lst]
        # sees
        lst = syn.also_sees()
        self.sees = [entry.name() for entry in lst]
        # domains
        lst = syn.topic_domains()
        self.domains = [
            entry.name() for entry in lst if entry.name() not in self.domains
        ]
        # root
        rn = syn.root_hypernyms()
        if len(rn) > 0:
            for rentry in rn:
                if len(self.root) > 1:
                    self.root += "/"
                self.root += str(rentry.name()).split(".")[0]
        # full tree
        lstm = syn.hypernym_paths()
        for li, lst in enumerate(lstm):
            rtxt = "root#" + str(li)
            rtxtn = "root#" + str(li)
            for entry in lst:
                if len(rtxt) > 1:
                    rtxt += "."
                    rtxtn += "."
                rtxt += entry.name()
                rtxtn += str(entry.name()).split(".")[0]
            self.root_tree.append(rtxt)
            self.root_tree_names.append(rtxtn)
        else:
            self.type = "WOE"
            meta: Dict = syn.metadata()
            self.pos = syn.pos
            if "description" in meta.keys():
                self.description = meta["description"]
            # print("Metadata", syn.metadata())
            # print("Sense", syn.senses())

            self.name = syn.senses()[0].word().lemma().replace(" ", "_")
            for sense in syn.get_related("domain_topic"):
                if len(sense.words()) > 0:
                    self.domains.append(sense.words()[0].lemma())

            self.offset = 0
            if len(self.synset) < 1:
                sid: str = str(syn.id)
                sids = sid.split("-")
                if len(sids) > 0:
                    sid = sids[1]
                else:
                    sid = sid.replace("odenet-", "")
                    sid = sid.replace("oewn-", "")
                    sid = sid.replace("-n", "")
                self.synset = self.name + "." + syn.pos + "." + sid

            self.ssid = str(syn.id)
            # lemmas
            self.lemmas = syn.lemmas()

            # hypernyms
            lst = syn.get_related("hypernym")
            self.hypernyms.extend(
                [nentry.lemmas()[0] for nentry in lst if len(nentry.lemmas()) > 0]
            )
            self.hypernyms.extend(
                [nentry.words()[0].lemma() for nentry in lst if len(nentry.words()) > 0]
            )

            # hyponyms
            lst = syn.get_related("hyponym")
            self.hyponyms.extend(
                [nentry.lemmas()[0] for nentry in lst if len(nentry.lemmas()) > 0]
            )
            self.hyponyms.extend(
                [nentry.words()[0].lemma() for nentry in lst if len(nentry.words()) > 0]
            )

            # holonyms
            lst = syn.get_related("holo_member")
            self.holonyms.extend(
                [nentry.lemmas()[0] for nentry in lst if len(nentry.lemmas()) > 0]
            )
            self.holonyms.extend(
                [nentry.words()[0].lemma() for nentry in lst if len(nentry.words()) > 0]
            )

            # meronyms
            lst = syn.get_related("mero_member")
            self.meronyms.extend(
                [nentry.lemmas()[0] for nentry in lst if len(nentry.lemmas()) > 0]
            )
            self.meronyms.extend(
                [nentry.words()[0].lemma() for nentry in lst if len(nentry.words()) > 0]
            )

            # entailments
            lst = syn.get_related("entails")
            self.entailments.extend(
                [nentry.lemmas()[0] for nentry in lst if len(nentry.lemmas()) > 0]
            )
            self.entailments.extend(
                [nentry.words()[0].lemma() for nentry in lst if len(nentry.words()) > 0]
            )

            # root
            rn = syn.hypernyms()
            if len(rn) > 0:
                for nentry in rn:
                    if len(self.root) > 1:
                        self.root += "/"
                    self.root += str(nentry.id)
            # full tree
            lstm = syn.hypernym_paths()
            for li, lst in enumerate(lstm):
                if li > 5:
                    break
                rtxt = "root#" + str(li)
                rtxtn = "root#" + str(li)
                for nentry in lst:
                    if len(rtxt) > 1:
                        if not rtxt.endswith("."):
                            rtxt += "."
                            rtxtn += "."
                    rtxt += nentry.id
                    if len(nentry.lemmas()) > 0:
                        rtxtn += str(nentry.lemmas()[0])
                    elif len(nentry.words()) > 0:
                        rtxtn += str(nentry.words()[0].lemma())
                self.root_tree.append(rtxt)
                self.root_tree_names.append(rtxtn)

        if len(self.description) < 1:
            self.description = syn.definition()

        # name
        ns = self.synset.split(".")
        if len(ns) > 0 and len(self.name) < 1:
            self.name = ns[0]
        elif len(self.name) < 1:
            self.name = self.synset

        if len(self.lexname) < 1:
            self.lexname = "generic"

        self.tree.append(self.lexname)
        self.tree.append(self.root)
        self.tree.extend(self.domains)
        self.tree.extend(self.hypernyms)

        self.synset = self.synset.upper()
        if not self.description:
            self.description = self.name.capitalize() + " > " + self.token.capitalize()

    def get_tree(self) -> str:
        ret: str = self.get_synset_list_names(self.tree, sep=".")
        return ret

    def get_root_tree_names(self) -> str:
        ret: str = ""
        for entry in self.root_tree_names:
            if len(ret) > 1:
                ret += "\n"
            ret += entry
        return ret

    def get_root_tree(self) -> str:
        ret: str = ""
        for entry in self.root_tree:
            if len(ret) > 1:
                ret += "\n"
            ret += entry
        return ret

    def get_frames(self) -> str:
        ret: str = self.get_synset_list_names(self.frames)
        return ret

    def get_frames_set(self) -> str:
        ret: str = self.get_synset_list_names(self.frames_set)
        return ret

    def get_domains(self) -> str:
        ret: str = self.get_synset_list_names(self.domains)
        return ret

    def get_lemmas(self) -> str:
        ret: str = self.get_synset_list_names(self.lemmas)
        return ret

    def get_sees(self) -> str:
        ret: str = self.get_synset_list_names(self.sees)
        return ret

    def get_similar(self) -> str:
        ret: str = self.get_synset_list_names(self.similar)
        return ret

    def get_hyponyms(self) -> str:
        ret: str = self.get_synset_list_names(self.hyponyms)
        return ret

    def get_hypernyms(self) -> str:
        ret: str = self.get_synset_list_names(self.hypernyms)
        return ret

    def get_holonyms(self) -> str:
        ret: str = self.get_synset_list_names(self.holonyms)
        return ret

    def get_meronyms(self) -> str:
        ret: str = self.get_synset_list_names(self.meronyms)
        return ret

    def get_entailments(self) -> str:
        ret: str = self.get_synset_list_names(self.entailments)
        return ret

    def get_synset_list_names(self, lst_syn: List, sep: str = ", ") -> str:
        ret: str = ""
        entry: str
        for entry in lst_syn:
            ln = entry.split(".")
            if len(ln) > 0:
                if len(ret) > 1:
                    ret += sep
                ret += ln[0].replace("_", " ")
            else:
                if len(ret) > 1:
                    ret += sep
                ret += entry.replace("_", " ")
        return ret

    def get_tooltip(self) -> str:
        ret = (
            "<span><i><small><strong>"
            + html.escape(self.type.upper() + "." + self.synset.upper())
            + "</strong> / "
            + html.escape(self.ssid.upper())
            + "</small></i><hr></span>"
        )  # <p>{{ entry[3] }}</p>
        ret += (
            "<p><h5><b><strong>"
            + html.escape(self.name.capitalize())
            + "</strong></b> > "
            + html.escape(self.token.capitalize())
            + "</h5></p>"
        )  # <p>{{ entry[3] }}</p>
        # ret += "<hr>"
        ret += (
            "<span><i><small><strong>Description</strong></small></i><hr><i><h6>"
            + html.escape(self.description.capitalize())
            + "<h6></i></span>"
        )
        # ret += "<hr>"
        ret += (
            "<span><i><small><strong>Node</strong></small></i><hr><h6>"
            + html.escape(self.get_tree())
            + "</h6></span>"
        )
        ret += (
            "<span><i><small><strong>Tree</strong></small></i><hr><h6>"
            + html.escape(self.get_root_tree_names()).replace("\n", "<br><br>")
            + "</h6></span>"
        )
        # ret += "<hr>"
        if len(self.sees) > 0:
            ret += (
                "<span><i><small><strong>Sees</strong></small></i><hr><h6>"
                + html.escape(self.get_sees())
                + "</h6></span>"
            )
        if len(self.similar) > 0:
            ret += (
                "<span><i><small><strong>Similar</strong></small></i><hr><h6>"
                + html.escape(self.get_similar())
                + "</h6></span>"
            )
        # ret += "<hr>"
        if len(self.hyponyms) > 0:
            ret += (
                "<span><i><small><strong>Specifics</strong></small></i><hr><h6>"
                + html.escape(self.get_hyponyms())
                + "</h6></span>"
            )
        if len(self.hypernyms) > 0:
            ret += (
                "<span><i><small><strong>Generalized</strong></small></i><hr><h6>"
                + html.escape(self.get_hypernyms())
                + "</h6></span>"
            )
        if len(self.holonyms) > 0:
            ret += (
                "<span><i><small><strong>Whole</strong></small></i><hr><h6>"
                + html.escape(self.get_holonyms())
                + "</h6></span>"
            )
        if len(self.meronyms) > 0:
            ret += (
                "<span><i><small><strong>Part</strong></small></i><hr><h6>"
                + html.escape(self.get_meronyms())
                + "</h6></span>"
            )
        # ret += "<hr>"
        if len(self.entailments) > 0:
            ret += (
                "<span><i><small><strong>Entailments</strong></small></i><hr><h6>"
                + html.escape(self.get_entailments())
                + "</h6></span>"
            )
        # ret += "<hr>"
        if len(self.domains) > 0:
            ret += (
                "<span><i><small><strong>Topic/Domain</strong></small></i><hr><h6>"
                + html.escape(self.get_domains())
                + "</h6></span>"
            )
        if len(self.frames) > 0:
            ret += (
                "<span><i><small><strong>Frames</strong></small></i><hr><h6>"
                + html.escape(self.get_frames())
                + "</h6></span>"
            )
        if len(self.frames_set) > 0:
            ret += (
                "<span><i><small><strong>FramesSet</strong></small></i><hr><h6>"
                + html.escape(self.get_frames_set())
                + "</h6></span>"
            )
        ret += (
            "<span><i><small><strong>Usual Words</strong></small></i><hr><h6><em>"
            + html.escape(self.get_lemmas())
            + "</h6></span>"
        )
        # ret = ret.replace("'", "\"")
        return ret


class WDCSpan(BaseModel):  # Used for entity
    id: int = -1
    text: str = ""
    ntokens: int = 0
    npos: int = 0
    type: str = ""
    misc: Dict = {}
    tokens: List[WDCToken] = []
    positions: List[WDCPosition] = []

    def addPosition(self, pos: WDCPosition):
        if pos:
            pos.id = len(self.positions)
            self.positions.append(pos)
            self.npos = len(self.positions)
            self.positions.sort(key=lambda x: (x.pageid, x.paraid, x.senid, x.s, x.e))
            for xi, pos in enumerate(self.positions):
                pos.id = xi


class WDCMLEntry(BaseModel):
    id: int = -1  #
    text: str = ""  #
    text_en: str = ""  #
    sentiment: str = ""
    density: float = 0.0

    en_xpos: List[str] = []
    en_upos: List[str] = []
    en_tokens: List[str] = []

    lng_xpos: List[str] = []
    lng_upos: List[str] = []
    lng_tokens: List[str] = []

    misc: Dict = {}

    nentities: int = 0
    nmeanings: int = 0

    entities: List[WDCSpan] = []
    meanings: List[WDCMeaning] = []

    def addEntity(self, entity: WDCSpan):
        if entity:
            self.entities.append(entity)
            self.nentities = len(self.entities)

    def addMeaning(self, meaning: WDCMeaning, pos: WDCPosition):
        if meaning:
            for mentry in self.meanings:
                if mentry.synset.__eq__(meaning.synset):
                    mentry.hits += 1
                    if pos not in mentry.positions:
                        mentry.positions.append(pos)
                    return

            self.nmeanings = len(self.meanings)
            meaning.id = self.nmeanings - 1
            meaning.positions.append(pos)
            self.meanings.append(meaning)


class WDCSentence(BaseModel):
    id: int = -1  #
    text: str = ""  #
    text_en: str = ""  #
    sentiment: str = ""
    density: float = 0.0
    ntokens: int = 0
    nwords: int = 0
    nentities: int = 0
    ndepends: int = 0
    ntriples: int = 0

    en_xpos: List[str] = []
    en_upos: List[str] = []
    en_tokens: List[str] = []

    lng_xpos: List[str] = []
    lng_upos: List[str] = []
    lng_tokens: List[str] = []

    tokens: List[WDCToken] = []
    triples: List[WDCTriple] = []
    dependencies: List = []
    misc: Dict = {}
    position: WDCPosition = WDCPosition()

    def addPosition(self, pos: WDCPosition):
        if pos:
            pos.senid = self.id
            self.position = pos

    def addToken(self, tok: WDCToken):
        if tok:
            tok.position.senid = self.id
            self.tokens.append(tok)
            self.ntokens = len(self.tokens)
            tok.id = self.ntokens - 1
            if self.density == 0.0:
                self.density = tok.density
            self.density = round(((self.density + tok.density) / 2.0), 2)

    def addTriple(self, triple: WDCTriple):
        if triple:
            triple.position.senid = self.id
            self.triples.append(triple)
            self.ntriples = len(self.triples)
            triple.id = self.ntriples - 1


class WDCParagraph(BaseModel):
    id: int = -1
    content: str = ""
    semantic: str = ""
    nsentences: int = 0
    ntokens: int = 0
    nwords: int = 0
    nentities: int = 0
    ndepends: int = 0

    lang: Optional[str] = None
    sentences: List[WDCSentence] = []
    position: WDCPosition = WDCPosition()

    def addPosition(self, pos: WDCPosition):
        if pos:
            pos.paraid = self.id
            self.position = pos

    def addSentence(self, sentence: WDCSentence):
        if sentence:
            sentence.position.paraid = self.id
            self.sentences.append(sentence)
            self.nsentences = len(self.sentences)
            sentence.id = self.nsentences - 1

    def getTextNoLF(self):
        ret = ""

        for sen in self.sentences:
            ret += sen.text + " "
        return ret


class WDCPage(BaseModel):
    id: int = -1
    content: str = ""
    nparagraphs: int = 0
    paragraphs: List[WDCParagraph] = []
    s: int = -1
    e: int = -1

    def addParagraph(self, paragraph: WDCParagraph):
        if paragraph:
            paragraph.position.pageid = self.id
            self.paragraphs.append(paragraph)
            self.nparagraphs = len(self.paragraphs)


class WDCMLDocument(BaseModel):
    content: str = ""
    nentries: int = 0
    targetsList: List = []
    trainList: List = []
    entries: List[WDCMLEntry] = []  #
    raw_json: List[Dict] = []
    df_data: List = []
    profile_html: str = ""
    profile_learn_html: str = ""
    prediction_html: str = ""
    leaderboard_html: str = ""

    def set_leaderboard_html(self, strHTML: str):
        if len(strHTML) > 0:
            self.leaderboard_html = html.escape(strHTML)

    def get_leaderboard_html(self) -> str:
        ret: str = ""
        if len(self.leaderboard_html) > 0:
            ret = html.unescape(self.leaderboard_html)
        return ret

    def set_profile_learn_html(self, strHTML: str):
        if len(strHTML) > 0:
            self.profile_learn_html = html.escape(strHTML)

    def get_profile_learn_html(self) -> str:
        ret: str = ""
        if len(self.profile_learn_html) > 0:
            ret = html.unescape(self.profile_learn_html)
        return ret

    def set_profile_html(self, strHTML: str):
        if len(strHTML) > 0:
            self.profile_html = html.escape(strHTML)

    def get_profile_html(self) -> str:
        ret: str = ""
        if len(self.profile_html) > 0:
            ret = html.unescape(self.profile_html)
        return ret

    def set_prediction_html(self, strHTML: str):
        if len(strHTML) > 0:
            self.prediction_html = html.escape(strHTML)

    def get_prediction_html(self) -> str:
        ret: str = ""
        if len(self.prediction_html) > 0:
            ret = html.unescape(self.prediction_html)
        return ret

    def addEntry(self, entry: WDCMLEntry):
        if entry:
            self.entries.append(entry)
            self.nentries = len(self.entries)


class WDCDocument(BaseModel):
    content: str = ""
    npages: int = 0  #
    nparagraphs: int = 0
    nsentences: int = 0
    ntokens: int = 0
    nwords: int = 0
    nentities: int = 0
    ndepends: int = 0
    ntriples: int = 0
    nmeanings: int = 0

    pages: List[WDCPage] = []  #
    entities: List[WDCSpan] = []
    meanings: List[WDCMeaning] = []

    def getCurrentPageID(self):
        ret = -1
        ix = len(self.pages)
        if ix > 0:
            ret = self.pages[ix - 1].id
        return ret

    def addPage(self, page: WDCPage):
        if page:
            self.pages.append(page)
            self.npages = len(self.pages)
            page.id = self.npages - 1

    def addEntity(self, entity: WDCSpan):
        if entity:
            self.entities.append(entity)
            self.nentities = len(self.entities)

    def addMeaning(self, meaning: WDCMeaning, pos: WDCPosition):
        if meaning:
            for mentry in self.meanings:
                if mentry.synset.__eq__(meaning.synset):
                    mentry.hits += 1
                    if pos not in mentry.positions:
                        mentry.positions.append(pos)
                    return

            self.nmeanings = len(self.meanings)
            meaning.id = self.nmeanings - 1
            meaning.positions.append(pos)
            self.meanings.append(meaning)



from typing import List, Optional

from pydantic import BaseModel


class MSAOpenAPIInfo(BaseModel):
    """
    **MSAOpenAPIInfo** Pydantic Response Class
    """

    name: str = "msaSDK Service"
    """Service Name."""
    version: str = "0.0.0"
    """API Version."""
    url: str = "/openapi.json"
    """OpenAPI URL."""
    tags: Optional[List[str]] = None
    """OpenAPI Tags."""



# -*- coding: utf-8 -*-

from typing import List

from pydantic import BaseModel


class WFLModule(BaseModel):
    id: int = -1
    short: str = ""
    name: str = ""
    description: str = ""
    step: str = "00000"

    class Config:
        orm_mode = False


class WFLNodeType(BaseModel):
    id: int = -1
    short: str = ""
    name: str = ""
    description: str = ""
    grp: str = "00000"
    icon: str = "input"

    class Config:
        orm_mode = False


list_node_modules: List[WFLModule] = []
list_node_types: List[WFLNodeType] = []

list_node_modules.append(
    WFLModule(id=1, short="collect", name="Collect", description="", step="10000")
)
list_node_modules.append(
    WFLModule(id=2, short="convert", name="Convert", description="", step="20000")
)
list_node_modules.append(
    WFLModule(
        id=3, short="preprocessing", name="Pre-Processing", description="", step="30000"
    )
)
list_node_modules.append(
    WFLModule(id=4, short="processing", name="Processing", description="", step="50000")
)
list_node_modules.append(
    WFLModule(
        id=5,
        short="postprocessing",
        name="Post-Processing",
        description="",
        step="70000",
    )
)
list_node_modules.append(
    WFLModule(id=6, short="format", name="Format", description="", step="80000")
)
list_node_modules.append(
    WFLModule(id=7, short="push", name="Push", description="", step="90000")
)

list_node_types.append(
    WFLNodeType(id=1, short="input", name="Input", description="", step="10000")
)
list_node_types.append(
    WFLNodeType(id=2, short="triggers", name="Triggers", description="", step="20000")
)
list_node_types.append(
    WFLNodeType(
        id=3, short="processors", name="Processors", description="", step="30000"
    )
)
list_node_types.append(
    WFLNodeType(id=4, short="operators", name="Operators", description="", step="50000")
)
list_node_types.append(
    WFLNodeType(id=5, short="actions", name="Actions", description="", step="70000")
)
list_node_types.append(
    WFLNodeType(id=6, short="models", name="Models", description="", step="80000")
)
list_node_types.append(
    WFLNodeType(id=7, short="output", name="Output", description="", step="90000")
)



import glob
from os.path import basename, dirname, isfile, join

version = "0.0.41"
__author__ = "Stefan Welcker"
__copyright__ = "Copyright 2022, U2D.ai"
__license__ = "MIT"
__version__ = version
__maintainer__ = "Stefan Welcker"
__email__ = "stefan@u2d.ai"
__status__ = "Beta"
__url__ = "https://github.com/swelcker/msaDocModels"

modules = glob.glob(join(dirname(__file__), "*.py"))
__all__ = [
    basename(f)[:-3] for f in modules if isfile(f) and not f.endswith("__init__.py")
]



#!/usr/bin/env python
from setuptools import find_packages, setup

# Parse version number from msaDocModels/__init__.py:
with open("msaDocModels/__init__.py") as f:
    info = {}
    for line in f:
        if line.startswith("version"):
            exec(line, info)
            break


setup_info = dict(
    name="msaDocModels",
    version=info["version"],
    author="Stefan Welcker",
    author_email="stefan@u2d.ai",
    url="https://github.com/swelcker/msaDocModels",
    download_url="http://pypi.python.org/pypi/msaDocModels",
    project_urls={
        "Documentation": "http://msaDocModels.u2d.ai/",
        "Source": "https://github.com/swelcker/msaDocModels",
        "Tracker": "https://github.com/swelcker/msaDocModels/issues",
    },
    description="msaDocModels - MSA Document Pydantic Models and Schemas, used to store Parser, NLP, NLU and AI results for processed documents",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    license="MIT",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Framework :: FastAPI",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Database",
        "Topic :: Database :: Front-Ends",
        "Topic :: Documentation",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Server",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    # Package info
    packages=["msaDocModels"]
    + ["msaDocModels." + pkg for pkg in find_packages("msaDocModels")],
    # Add _ prefix to the names of temporary build dirs
    options={
        "build": {"build_base": "_build"},
    },
    zip_safe=True,
)

setup(**setup_info)



import datetime
from typing import Any, List, Optional, Union

from pydantic import BaseModel


class MSASchedulerRepoLogRecord(BaseModel):
    """**MSASchedulerRepoLogRecord** Pydantic Response Class"""

    task_name: str
    action: str
    created: Optional[datetime.datetime]
    name: str
    msg: str
    levelname: Any
    levelno: int
    pathname: str
    filename: str
    module: str
    exc_text: Any
    lineno: int
    funcName: str
    msecs: float
    relativeCreated: Optional[datetime.datetime]
    thread: int
    threadName: str
    processName: str
    process: int
    message: Any
    formatted_message: Any


class MSASchedulerTaskDetail(BaseModel):
    """**MSASchedulerTaskDetail** Pydantic Response Class"""

    permanent_task: bool
    fmt_log_message: str
    daemon: Any
    name: str
    description: Any
    logger_name: str
    execution: Any
    priority: int
    disabled: bool
    force_run: bool
    force_termination: bool
    status: str
    timeout: Optional[Union[str, int, datetime.timedelta]]
    parameters: Any
    start_cond: Any
    end_cond: Any
    on_startup: bool
    on_shutdown: bool
    last_run: Optional[datetime.datetime]
    last_success: Optional[datetime.datetime]
    last_fail: Optional[datetime.datetime]
    last_terminate: Optional[datetime.datetime]
    last_inaction: Optional[datetime.datetime]
    last_crash: Optional[datetime.datetime]
    func: Any
    path: Any
    func_name: str
    cache: bool
    sys_paths: List


class MSASchedulerTaskStatus(BaseModel):
    """**MSASchedulerTaskStatus** Pydantic Response Class"""

    name: Optional[str] = None
    """Task Name."""
    detail: Optional[MSASchedulerTaskDetail] = None
    """Task detail."""


class MSASchedulerStatus(BaseModel):
    """
    **MSASchedulerStatus** Pydantic Response Class
    """

    name: Optional[str] = "msaSDK Service"
    """Service Name."""
    tasks: Optional[List[MSASchedulerTaskStatus]] = []
    """Optional MSASchedulerTaskStatus List"""
    message: Optional[str] = "None"
    """Optional Message Text"""


class MSASchedulerLog(BaseModel):
    """
    **MSASchedulerStatus** Pydantic Response Class
    """

    name: Optional[str] = "msaSDK Service"
    """Service Name."""
    log: Optional[List[MSASchedulerRepoLogRecord]] = []
    """Optional MSASchedulerRepoLogRecord List"""
    message: Optional[str] = "None"
    """Optional Message Text"""
